import 'dart:async';

import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import 'package:puzzlegame/provider/data.dart';
import 'package:puzzlegame/screens/game.dart';
import 'package:puzzlegame/utils/game_utils.dart';
import 'package:puzzlegame/utils/get_images_list.dart';

class Home extends StatelessWidget {
  const Home({super.key});

  static const String id = 'home';

  final Color themePrimaryColor = Colors.blue;
  final int _selectImageColumnsCount = 3;

  Future<void> getImagesList(Data myProvider) async {
    GetImagesList getImagesList = GetImagesList();
    await getImagesList.init();
    if (getImagesList.availableImages.isNotEmpty) {
      myProvider.updateAvailableImages(getImagesList.availableImages);
      shuffleAvailableImages(myProvider);
    }
  }

  Future<void> selectImage(Data myProvider, String imageCode) async {
    GameUtils.startGame(myProvider, imageCode);
  }

  void shuffleAvailableImages(Data myProvider) {
    final List<String> images = myProvider.availableImages;
    images.shuffle();
    myProvider.updateAvailableImages(images);
  }

  Widget _buildImageSelectorItem(Data myProvider, String image) {
    return TextButton(
      style: TextButton.styleFrom(
        padding: const EdgeInsets.all(2),
      ),
      child: Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(4),
          border: Border.all(
            color: Colors.blue.shade200,
            width: 4,
          ),
        ),
        child: Image(
          image: AssetImage(GameUtils.getImageAssetName(image)),
          fit: BoxFit.fill,
        ),
      ),
      onPressed: () {
        selectImage(myProvider, image);
      },
    );
  }

  Widget _buildImageSelector(Data myProvider) {
    if (myProvider.availableImages.isEmpty) {
      getImagesList(myProvider);
    }
    final List<String> images = myProvider.availableImages;

    return Container(
      padding: const EdgeInsets.all(2),
      child: ListView(
        children: [
          Table(
            defaultColumnWidth: const IntrinsicColumnWidth(),
            children: [
              for (int imgIndex = 0;
                  imgIndex < images.length;
                  imgIndex += _selectImageColumnsCount)
                TableRow(
                  children: [
                    for (int colIndex = 0; colIndex < _selectImageColumnsCount; colIndex++)
                      Column(
                        children: [
                          if (imgIndex + colIndex < images.length)
                            _buildImageSelectorItem(myProvider, images[imgIndex + colIndex])
                        ],
                      ),
                  ],
                ),
            ],
          ),
        ],
      ),
    );
  }

  TextButton _buildTilesetSizeSelectorItem(Data myProvider, int value) {
    String assetName = 'assets/icons/difficulty_${value}x$value.png';

    Color borderColor = themePrimaryColor;

    if (myProvider.tilesCount == value) {
      borderColor = Colors.white;
    }

    return TextButton(
      child: Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(4),
          border: Border.all(
            color: borderColor,
            width: 4,
          ),
        ),
        child: Image(
          image: AssetImage(assetName),
          fit: BoxFit.fill,
        ),
      ),
      onPressed: () {
        myProvider.updateTilesCount(value);
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    final Data myProvider = Provider.of<Data>(context);

    Widget content;

    myProvider
        .updateTileImageSize((MediaQuery.of(context).size.width - 70) / myProvider.tilesCount);

    if (myProvider.isShufflingBoard) {
      content = Game.buildShufflingIndicatorWidget();
      precacheImage(const AssetImage('assets/icons/game_win.png'), context);
    } else {
      if (myProvider.selectedImage == '') {
        content = _buildImageSelector(myProvider);
      } else {
        content = Game.buildGameWidget(myProvider);
      }
    }

    final List<Widget> menuActions = [
      _buildTilesetSizeSelectorItem(myProvider, 3),
      _buildTilesetSizeSelectorItem(myProvider, 4),
      _buildTilesetSizeSelectorItem(myProvider, 5),
      TextButton(
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(4),
            border: Border.all(
              color: themePrimaryColor,
              width: 4,
            ),
          ),
          child: const Image(
            image: AssetImage('assets/icons/button_shuffle.png'),
            fit: BoxFit.fill,
          ),
        ),
        onPressed: () => shuffleAvailableImages(myProvider),
      ),
      TextButton(
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(4),
            border: Border.all(
              color: themePrimaryColor,
              width: 4,
            ),
          ),
          child: const Image(
            image: AssetImage('assets/icons/button_random_pick.png'),
            fit: BoxFit.fill,
          ),
        ),
        onPressed: () => GameUtils.startRandomGame(myProvider),
      ),
    ];

    final List<Widget> gameActions = [
      TextButton(
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(4),
            border: Border.all(
              color: Colors.blue,
              width: 4,
            ),
          ),
          child: const Image(
            image: AssetImage('assets/icons/button_back.png'),
            fit: BoxFit.fill,
          ),
        ),
        onPressed: () => GameUtils.resetGame(myProvider),
      ),
    ];

    return Scaffold(
      appBar: AppBar(
        backgroundColor: themePrimaryColor,
        actions: myProvider.selectedImage == '' ? menuActions : gameActions,
      ),
      body: SafeArea(
        child: Center(child: content),
      ),
    );
  }
}
